@@ -1,6 +1,6 @@
 ;;; tex-info.el --- Support for editing Texinfo source.  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1993-2021  Free Software Foundation, Inc.
+;; Copyright (C) 1993-2023  Free Software Foundation, Inc.
 
 ;; Maintainer: auctex-devel@gnu.org
 ;; Keywords: tex
@@ -31,9 +31,8 @@
 
 (require 'texinfo)
 
-;; Silence the compiler for Emacs compiled `--with-native-compilation':
-(declare-function TeX-TeX-sentinel "tex-buf"
-                  (process name))
+;; Silence the compiler for variables:
+(defvar outline-heading-alist)
 
 ;;; Environments:
 (defvar Texinfo-environment-list
@@ -168,7 +167,7 @@ there are unbalanced begin-end pairs in comments and verbatim
 environments."
   ;; TODO:
   ;; This is identical to the LaTeX counterpart but for the find begin/end
-  ;; functions. So some day the implemenation should be factorized.
+  ;; functions.  So some day the implemenation should be factorized.
   (interactive "p")
   (setq count (if count (abs count) 1))
   (let ((cur (point)) beg end)
@@ -294,7 +293,7 @@ beginning of keyword `@node' or `@bye'."
 
 (defun Texinfo-nodename-de-escape (node-name)
   "In NODE-NAME, convert `@comma{}' commands to the corresponding `,'
-character. Return the resulting string."
+character.  Return the resulting string."
   (let ((pos 0) (map '(("comma" . ","))))
     (while (and (< pos (length
                         node-name))
@@ -308,7 +307,7 @@ character. Return the resulting string."
 
 (defun Texinfo-nodename-escape (node-name)
   "Convert in NODE-NAME the `,' characters to `@comma{}'
-commands. Return the resulting string."
+commands.  Return the resulting string."
   (let* ((pos 0)
          (map '(("," . "comma")))
          (re (regexp-opt (mapcar #'car map))) )
@@ -403,7 +402,7 @@ for @node."
 OPTIONAL is ignored.
 Use PROMPT as the prompt string.
 If DEFINITION is non-nil, then chosen node name is a node name to be
-added to the list of defined node names. Current implementation
+added to the list of defined node names.  Current implementation
 ignored DEFINITION as the full document is scanned for node names at
 each invocation."
   (let ((node-name (completing-read (TeX-argument-prompt optional prompt "Node")
@@ -470,7 +469,7 @@ is assumed by default."
          (literal (buffer-substring-no-properties
                    (1- (match-beginning 3))
                    (min (point-max) (+ (match-end 0) (length text1) 1))))
-         ;; Literal can be too short since text1 too short. No big problem.
+         ;; Literal can be too short since text1 too short.  No big problem.
          (text (reftex-nicify-text text1)))
 
     ;; Add section number and indentation
@@ -480,26 +479,17 @@ is assumed by default."
            (if (nth 1 reftex-label-menu-flags) ; section number flag
                (concat section-number " "))
            text))
-    (prog1
-        (list 'toc "toc" text file marker level section-number
-              literal (marker-position marker))
-      (set-marker marker nil))))
+    (list 'toc "toc" text file marker level section-number
+          literal (marker-position marker))))
 
 (defun Texinfo-reftex-hook ()
   "Hook function to plug Texinfo into RefTeX."
   ;; force recompilation of variables
-  (when (string= TeX-base-mode-name "Texinfo")
-    ;; dirty temporary hook to remove when reftex has a Texinfo builtin
-    ;; TODO --- taken on <2014-01-06 mon> --- remove the dirty trick once reftex
-    ;; has been corrected for long enough a time
-    (unless (assq 'Texinfo reftex-label-alist-builtin)
-      (setq reftex-label-alist-builtin (append reftex-label-alist-builtin
-                                               '((Texinfo "Texinfo default environments" nil)))))
-    (dolist (v `((reftex-section-pre-regexp . "@")
-                 ; section post-regexp must contain exactly one group
-                 (reftex-section-post-regexp . "\\([ \t]+\\)")
-                 (reftex-section-info-function . Texinfo-reftex-section-info)
-                 (reftex-default-label-alist-entries . (Texinfo))
+  (dolist (v `((reftex-section-pre-regexp . "@")
+               ;; section post-regexp must contain exactly one group
+               (reftex-section-post-regexp . "\\([ \t]+\\)")
+               (reftex-section-info-function . Texinfo-reftex-section-info)
+               (reftex-default-label-alist-entries . (Texinfo))
                (reftex-section-levels
                 . ,(mapcar
                     (lambda (x)
@@ -508,8 +498,8 @@ is assumed by default."
                           (cons (car x) (- (cadr x)))
                         (cons (car x) (cadr x))))
                     texinfo-section-list))))
-      (set (make-local-variable (car v) ) (cdr v)))
-    (reftex-ensure-compiled-variables)))
+    (set (make-local-variable (car v) ) (cdr v)))
+  (reftex-ensure-compiled-variables))
 
 ;;; Keymap:
 
@@ -524,6 +514,7 @@ is assumed by default."
     (define-key map "\C-c\C-u\C-n"   #'texinfo-update-node)
     (define-key map "\C-c\C-u\C-e"   #'texinfo-every-node-update)
     (define-key map "\C-c\C-u\C-a"   #'texinfo-all-menus-update)
+    (define-key map "\C-c\C-u\C-d"   #'texinfo-start-menu-description)
 
     ;; Simulating LaTeX-mode
     (define-key map "\C-c\C-e" #'Texinfo-environment)
@@ -547,19 +538,22 @@ is assumed by default."
     (define-key map "\\" #'self-insert-command)
     ;; Users benefit from `TeX-electric-macro' even in Texinfo mode
     (define-key map "@" #'TeX-insert-backslash)
+
+    ;; Hide "Text" menu entry inherited from text mode.
+    (define-key map [menu-bar text] #'undefined)
     map)
   "Keymap for Texinfo mode.")
 
 (easy-menu-define Texinfo-command-menu
   Texinfo-mode-map
   "Menu used in Texinfo mode for external commands."
-  (TeX-mode-specific-command-menu 'texinfo-mode))
+  (TeX-mode-specific-command-menu 'Texinfo-mode))
 
 (easy-menu-define Texinfo-mode-menu
   Texinfo-mode-map
   "Menu used in Texinfo mode."
   `("Texinfo"
-    ["Node ..." texinfo-insert-@node
+    ["Node ..." Texinfo-insert-node
      :help "Insert a node"]
     ["Macro ..." TeX-insert-macro
      :help "Insert a macro and possibly arguments"]
@@ -631,37 +625,33 @@ is assumed by default."
 
 ;;; Mode:
 
-;;;###autoload
-(defalias 'Texinfo-mode #'texinfo-mode)
-
-(defvar TeX-sentinel-default-function) ;; Defined in tex-buf.el.
+(defvar semantic-symref-filepattern-alist) ; Silence compiler
+(with-eval-after-load 'semantic/symref/grep
+  (push '(Texinfo-mode "*.texinfo" "*.texi" "*.txi")
+        semantic-symref-filepattern-alist))
 
 ;;;###autoload
-(defun TeX-texinfo-mode ()
+(define-derived-mode Texinfo-mode text-mode "Texinfo"
   "Major mode in AUCTeX for editing Texinfo files.
 
-Special commands:
-\\{Texinfo-mode-map}
-
 Entering Texinfo mode calls the value of `text-mode-hook' and then the
 value of `Texinfo-mode-hook'."
-  (interactive)
-  (kill-all-local-variables)
+  :syntax-table texinfo-mode-syntax-table
+  :after-hook (Texinfo-mode-cleanup)
+
   (setq TeX-mode-p t)
   (setq TeX-output-extension (if TeX-PDF-mode "pdf" "dvi"))
   (setq TeX-sentinel-default-function #'TeX-TeX-sentinel)
   ;; Mostly stolen from texinfo.el
-  (setq TeX-base-mode-name "Texinfo")
-  (setq major-mode 'texinfo-mode)
-  (use-local-map Texinfo-mode-map)
-  (set-syntax-table texinfo-mode-syntax-table)
+  (setq TeX-base-mode-name mode-name)
 
-  (set (make-local-variable 'page-delimiter)
-       (concat
-        "^@node [ \t]*[Tt]op\\|^@\\("
-        texinfo-chapter-level-regexp
-        "\\)"))
-  (set (make-local-variable 'require-final-newline) t)
+  ;; Moved after `run-mode-hooks'. (bug#65750)
+  ;; (set (make-local-variable 'page-delimiter)
+  ;;      (concat
+  ;;       "^@node [ \t]*[Tt]op\\|^@\\("
+  ;;       texinfo-chapter-level-regexp
+  ;;       "\\)"))
+  (set (make-local-variable 'require-final-newline) mode-require-final-newline)
   (set (make-local-variable 'indent-tabs-mode) nil)
   (set (make-local-variable 'paragraph-separate)
        (concat "@[a-zA-Z]*[ \n]\\|" paragraph-separate))
@@ -675,18 +665,20 @@ value of `Texinfo-mode-hook'."
   (set (make-local-variable 'words-include-escapes) t)
   (set (make-local-variable 'imenu-generic-expression)
        texinfo-imenu-generic-expression)
+  (setq imenu-case-fold-search nil)
 
   (set (make-local-variable 'font-lock-defaults)
        '(texinfo-font-lock-keywords nil nil nil backward-paragraph))
   (set (make-local-variable 'syntax-propertize-function)
        texinfo-syntax-propertize-function)
 
-  ;; Outline settings.
-  (set (make-local-variable 'outline-regexp)
-       (concat "@\\("
-               (mapconcat #'car texinfo-section-list "\\>\\|")
-               "\\>\\)"))
-  (set (make-local-variable 'outline-level) 'texinfo-outline-level)
+  ;; Moved after `run-mode-hooks'. (bug#65750)
+  ;; (setq-local outline-heading-alist
+  ;;             (mapcar (lambda (x) (cons (concat "@" (car x)) (cadr x)))
+  ;;       	      texinfo-section-list))
+  ;; (setq-local outline-regexp
+  ;;             (concat (regexp-opt (mapcar #'car outline-heading-alist) t)
+  ;;       	      "\\>"))
 
   ;; Mostly AUCTeX stuff
   (set (make-local-variable 'TeX-command-current) #'TeX-command-master)
@@ -697,12 +689,14 @@ value of `Texinfo-mode-hook'."
   (set (make-local-variable 'TeX-auto-regexp-list) 'TeX-auto-empty-regexp-list)
 
   (setq TeX-command-default "TeX")
-  (setq TeX-header-end "%*end")
-  (setq TeX-trailer-start (regexp-quote (concat TeX-esc "bye")))
+  (setq TeX-header-end (regexp-quote "%**end of header"))
+  (setq TeX-trailer-start (format "^%s$"
+                                  (regexp-quote (concat TeX-esc "bye"))))
 
-  (set (make-local-variable 'TeX-complete-list)
-       (list (list "@\\([a-zA-Z]*\\)" 1 'TeX-symbol-list-filtered nil)
-             (list "" TeX-complete-word)))
+  ;; Moved after `run-mode-hooks'. (bug#65750)
+  ;; (set (make-local-variable 'TeX-complete-list)
+  ;;      (list (list "@\\([a-zA-Z]*\\)" 1 #'TeX-symbol-list-filtered nil)
+  ;;            (list "" TeX-complete-word)))
 
   (set (make-local-variable 'TeX-font-list) Texinfo-font-list)
   (set (make-local-variable 'TeX-font-replace-function)
@@ -735,7 +729,7 @@ value of `Texinfo-mode-hook'."
    '("bye")
    '("c" (TeX-arg-literal " ") (TeX-arg-free "Comment"))
    '("caption" "Caption"
-     ;; TODO: caption is meaningful only inside float env. Maybe some checking
+     ;; TODO: caption is meaningful only inside float env.  Maybe some checking
      ;; and warning would be good.
      )
    '("center" (TeX-arg-literal " ") (TeX-arg-free "Line of text"))
@@ -863,18 +857,44 @@ value of `Texinfo-mode-hook'."
    '("xref" (Texinfo-arg-nodename "Node name")))
 
   ;; RefTeX plugging
-  (add-hook 'reftex-mode-hook #'Texinfo-reftex-hook)
+  (add-hook 'reftex-mode-hook #'Texinfo-reftex-hook nil t)
   (if (and (boundp 'reftex-mode) reftex-mode)
-      (Texinfo-reftex-hook))
+      (Texinfo-reftex-hook)))
+
+(defun Texinfo-mode-cleanup ()
+  "Cleanup function for `Texinfo-mode'.
+Run after mode hooks and file local variables application."
+  ;; Don't overwrite the value the user set by hooks or file
+  ;; (directory) variables.
+  (or (local-variable-p 'page-delimiter)
+      (setq-local page-delimiter
+                  (concat
+                   "^@node [ \t]*[Tt]op\\|^@\\("
+                   texinfo-chapter-level-regexp
+                   "\\)")))
+
+  ;; Outline settings.
+  (or (local-variable-p 'outline-heading-alist)
+      (setq-local outline-heading-alist
+	          (mapcar (lambda (x) (cons (concat "@" (car x)) (cadr x)))
+		          texinfo-section-list)))
+  (or (local-variable-p 'outline-regexp)
+      (setq-local outline-regexp
+	          (concat (regexp-opt (mapcar #'car outline-heading-alist) t)
+		          "\\>")))
+
+  (or (local-variable-p 'TeX-complete-list)
+      (setq-local TeX-complete-list
+                  (list (list "@\\([a-zA-Z]*\\)" 1 #'TeX-symbol-list-filtered nil)
+                        (list "" TeX-complete-word))))
 
-  (run-mode-hooks 'text-mode-hook 'Texinfo-mode-hook)
   (TeX-set-mode-name))
 
 (defcustom Texinfo-clean-intermediate-suffixes
   '("\\.cps?" "\\.vrs?" "\\.fns?" "\\.tps?" "\\.pgs?" "\\.kys?")
   "List of regexps matching suffixes of files to be deleted.
 The regexps will be anchored at the end of the file name to be matched,
-that is, you do _not_ have to cater for this yourself by adding \\\\' or $."
+that is, you do _not_ have to cater for this yourself by adding \\\\\\=' or $."
   :type '(repeat regexp)
   :group 'TeX-command)
 
@@ -884,7 +904,7 @@ that is, you do _not_ have to cater for this yourself by adding \\\\' or $."
     "_toc\\.html" "_fot\\.html" "_abt\\.html" "_[0-9]+\\.html" "_l2h_img.+")
   "List of regexps matching suffixes of files to be deleted.
 The regexps will be anchored at the end of the file name to be matched,
-that is, you do _not_ have to cater for this yourself by adding \\\\' or $."
+that is, you do _not_ have to cater for this yourself by adding \\\\\\=' or $."
   :type '(repeat regexp)
   :group 'TeX-command)
 
