@@ -1,6 +1,6 @@
 ;;; texmathp.el -- Code to check if point is inside LaTeX math environment  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1998-2021  Free Software Foundation, Inc.
+;; Copyright (C) 1998-2022  Free Software Foundation, Inc.
 
 ;; Author: Carsten Dominik <dominik@strw.LeidenUniv.nl>
 ;; Maintainer: auctex-devel@gnu.org
@@ -52,7 +52,7 @@
 ;;  mode being on or off.
 ;;
 ;;  To configure which macros and environments influence LaTeX math
-;;  mode, customize the variable `texmathp-tex-commands'. By default
+;;  mode, customize the variable `texmathp-tex-commands'.  By default
 ;;  it recognizes the plain TeX and LaTeX core as well as AMS-LaTeX
 ;;  and packages mathtools, empheq and breqn (see the variable
 ;;  `texmathp-tex-commands-default', also as an example).
@@ -85,14 +85,19 @@
 ;;
 ;;  BUGS:
 ;;
-;;  If any of the the special macros like \mbox or \ensuremath has optional
-;;  arguments, math mode inside these optional arguments is *not* influenced
-;;  by the macro.
+;;  o If any of the the special macros like \mbox or \ensuremath has
+;;    optional arguments, math mode inside these optional arguments is
+;;    *not* influenced by the macro.
 ;;
-;;  Nested \(\) and \[\] can confuse texmathp. It returns nil at AAA in the
-;;  following examples:
+;;  o Nested \(\) and \[\] can confuse texmathp.  It returns nil at AAA
+;;    in the following examples:
 ;;  \[ x=y \mbox{abc \(\alpha\) cba} AAA \]
 ;;  \[ x=y \begin{minipage}{3cm} abc \[\alpha\] cba \end{minipage} AAA \]
+;;
+;;  o In the "text column" of cases* environment, texmathp doesn't
+;;    consider it's non-math mode.  The same applies for variants of
+;;    cases* environents, both provided by mathtools package.
+;;
 ;;--------------------------------------------------------------------------
 
 ;;; Code:
@@ -275,10 +280,10 @@ See the variable `texmathp-tex-commands' about which commands are checked."
   (let* ((pos (point)) math-on sw-match
          (bound (save-excursion
                   (if (re-search-backward
-                       (if (eq major-mode 'doctex-mode)
+                       (if (memq major-mode '(doctex-mode docTeX-mode))
                            "[\n\r]%*[ \t]*[\n\r]"
                          "[\n\r][ \t]*[\n\r]")
-                                          nil 1 texmathp-search-n-paragraphs)
+                       nil 1 texmathp-search-n-paragraphs)
                       (match-beginning 0)
                     (point-min))))
          (mac-match (texmathp-match-macro bound))
@@ -316,12 +321,25 @@ See the variable `texmathp-tex-commands' about which commands are checked."
 
     ;; Store info, show as message when interactive, and return
     (setq texmathp-why match)
-    (and (called-interactively-p 'any)
-         (message "math-mode is %s: %s begins at buffer position %d"
-                  (if math-on "on" "off")
-                  (or (car match) "new paragraph")
-                  (cdr match)))
-    (and math-on t)))
+    ;; Check also if the match is inside a verbatim construct and
+    ;; return immediately nil.  This relies on the function
+    ;; `LaTeX-verbatim-p'.  We add a check here in case this library
+    ;; is used stand-alone without latex.el provided by AUCTeX
+    ;; (bug#61410):
+    (if (and (fboundp 'LaTeX-verbatim-p)
+             (LaTeX-verbatim-p (cdr match)))
+        (progn
+          (setq texmathp-why `(nil . ,(cdr match)))
+          (when (called-interactively-p 'any)
+            (message "math-mode is off: Math command in verbatim construct at buffer position %d"
+                     (cdr match)))
+          nil)
+      (and (called-interactively-p 'any)
+           (message "math-mode is %s: %s begins at buffer position %d"
+                    (if math-on "on" "off")
+                    (or (car match) "new paragraph")
+                    (cdr match)))
+      (and math-on t))))
 
 (defun texmathp-match-environment (bound)
   "Find out if point is inside any of the math environments.
